/*
* ATM168.c Code file to test working with ATMEGA168
*
*/

#undef F_CPU
#define F_CPU 16000000UL		//Your mileage will vary here. See InitMotors() for changes to TCCR0B prescale!
#define BAUD 57600L 
#define BAUD_RR (F_CPU/16L/BAUD-1) 

#include <avr/interrupt.h> 
#include <inttypes.h> 
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include "atm168.h"

/* Global stuff to use */
volatile uint8_t   	servo;						// My one and only servo
volatile uint32_t 	t_1ms;						// Background 1ms ticker
volatile uint8_t	edge;						// CIP1 capture edge
volatile uint16_t	pulse;						// our pulse width

FILE *fileDef;									// To use the printf() function
int def_putc(char ch,FILE *fileDef);
int def_getc(FILE *fileDef);

int main(void)
{
	uint16_t	lastPulse = 0;
	
	initialize();							// Initialize the board
	t_1ms = 0;								// init the clock
		
    // To invoke the stdin.h printf, uncomment below (REALLY BIG!)
    fileDef = fdevopen(def_putc, def_getc);     //init stdio

	PORTC = 0x00;
	
	printf("starting\n\r");
	waitms(2000);
	
	STATUS_LED = 1;
	servo = 150;							// center of servo

	while(1)
	{
		waitms(100);						// once every 100ms
		if ((abs(pulse - lastPulse)) > 100)
		{
			lastPulse = pulse;
			printf("%u\r",pulse);				// print out the pulse value		
		}
	}
}

/*
* ISRs and misc functions follow
*/
ISR(TIMER2_COMPA_vect) 
/*
 * 10 microsecond ISR on TIMER2, a 8 bit clock for servos and
 * background timer.
 */
{
	//Static vars are automatically initialized to 0 in .bss
	static uint8_t 		t_10us;
	static uint16_t  	match;			// The single servo used
	static uint8_t  	next_pulse;

	t_10us++;
    if (t_10us == 100)
    {
    	t_1ms++;
    	t_10us = 0;
    }
  
    match++;                   	// increment every 1
    if (match > next_pulse)
    	SPIN = 0;                // drop servo bit   
    else
    	SPIN = 1;                // raise servo pin high

    if (match > 1900) 
    {
    	next_pulse = servo;
        match = 0;                //start servo timer
    }
}

ISR(TIMER1_CAPT_vect )
{
	if (edge == 1) {
		TCNT1H = 0;
		TCNT1L = 0;				// we want the positive side pulse width
		edge = 0;
		TCCR1B &= ~(unsigned char)_BV(ICES1);	// Now look for falling edge
	}
	else {
		pulse = (uint8_t)ICR1L;
		pulse += (uint16_t)(ICR1H <<8);			// get width
		edge = 1;
		TCCR1B |= (unsigned char)_BV(ICES1);	// flip trigger sense
	}
}

void InitTimer1(void)
/*
 * Set up the ICP1 input compare interrupt system.
 * 
 */
{
	TCCR1A = 0x00;							// No Output compares
	TCCR1B = 0xC1;							// rising edge, noise canceler
	TCCR1C = 0x00;							// just to be thorough
	TCNT1H = 0x00;							// clear the counter
	TCNT1L = 0x00;
	
	edge = 1;								// flag rising edge half
	TIMSK1 |= (unsigned char)_BV(ICIE1);	// enable interrupt
}

void InitTimer2(void)
{
  /*
   * Initialize timer2 to generate an output compare interrupt, and
   * set the output compare register so that we get that interrupt
   * every 10us.  This is 95 tics, or load with 255-95 = 160, no prescale.
   */
	TIFR1 = 0;
    TCCR2A = 0x02;							// Timer match up on OC2A
    TCCR2B = 0x01; 							// raw clock, no prescale
    TCNT2  = 0;								// clear for startup
    TIMSK2 |= (unsigned char)_BV(OCIE2A);   // enable output compare interrupt
    OCR2A = 160;          					// match in 10us
    OCR2B = 0;								// don't interrupt here
}

void initialize()
/*
 * (unsigned char) prevents gcc from promoting a bitwise operator to a 16 bit word.
 */
{
	DDRC = 0xFE; 							// All outputs but C0 = AN0, analog input
	DDRB = 0xDE; 							// Set up the rest according to pinouts
	DDRD = 0xF2;							//PD0(RxD),PD2,PD3 (INT0 and INT1) now inputs

	ACSR = 0x80; 							// turn off the comparitor
	DIDR0 = 0x01;							// turn digital off for PC0
	ADCSRA = 0xE7;							// turn A2D on, prescale 128 for 125KHz sample rate
	ADMUX = 0x20;							// Left justify, MUX = AD0
	ADCSRB = 0x00;							// Free run A2D
		
    //Enable UART (BAUD set above)
    UBRR0H = (unsigned char)(BAUD_RR >> 8);
    UBRR0L = (unsigned char)BAUD_RR;
    UCSR0B = (unsigned char)_BV(TXEN0) | _BV(RXEN0);
    UCSR0C = (unsigned char)_BV(UCSZ01) | _BV(UCSZ00);
    //9600 baud, 8bit, 1 start bit, 1 stop bits: Rx and Tx
    
    InitTimer1();							//Set up input compare interrupt
    InitTimer2();							//Set up time keeper interrupt
    servo = 70;								//set up the servo initial value
    sei();                         			// enable interrupts
}


void waitms(unsigned int d)
/*
 * This just uses the background 1ms ticker to do its dirty work.
 */
{
	uint32_t delay;

	delay = (uint32_t)(t_1ms + d); 				//get initial count
	while (delay != t_1ms);						// wait until timer catches up

}

int def_putc(char ch,FILE *fileDef)
/*
 * The default putc routine for printf()
 */
 {
     while ((unsigned char)(UCSR0A & _BV(UDRE0)) == 0)
           ;
     UDR0 = ch;
     return ch;
 }

int def_getc(FILE *fileDef)
{
	//Crude, but workable
	if ((unsigned char)(UCSR0A & (1<<RXC0)))
		return (int)UDR0;
	else
		return (int)-1;
}
 
